/*

*/

import java.util.concurrent.locks.ReentrantLock;

/**
   This simulates a first-in-first-out (stack) buffer.
*/
public class Buffer_locked extends Buffer
{
   private final ReentrantLock lock;

   /**
      Set a default buffer size of 80.
   */
   Buffer_locked(final int currentIndex)
   {
      this(currentIndex, 80);
   }

   /**
      Create a buffer with a mutual exclusion lock.
   */
   Buffer_locked(final int currentIndex, final int SIZE)
   {
      super(currentIndex, SIZE); // Initialize a partially filled buffer.

      this.lock = new ReentrantLock();
   }


   @Override
   public void put(char c)
   {
      this.lock.lock(); // mutual exclusion
      try
      {
         ++currentIndex;
         Thread.yield(); // force race conditions to happen more often
         buf[currentIndex] = c;
      }
      finally
      {
         this.lock.unlock();
      }
   }


   @Override
   public void get(char c)
   {
      this.lock.lock(); // mutual exclusion
      try
      {
         buf[currentIndex] = c;
         Thread.yield(); // force race conditions to happen more often
         --currentIndex;
      }
      finally
      {
         this.lock.unlock();
      }
   }
}
