/*

*/

/**
   This simulates a first-in-first-out (stack) buffer.
*/
public class Buffer
{
   protected final int SIZE;
   protected final char[] buf;
   protected int currentIndex;

   /**
      Set a default buffer size of 80.
   */
   Buffer(final int currentIndex)
   {
      this(currentIndex, 80);
   }

   /**
      Initialize a partially filled buffer.
   */
   Buffer(final int currentIndex, final int SIZE)
   {
      // currentIndex == 0 initializes an empty buffer
      // currentIndex == SIZE initializes a full buffer
      if (currentIndex < 0 || currentIndex > SIZE)
         throw new Error("illegal buffer intialization");

      this.SIZE = SIZE;
      this.buf = new char[SIZE + 2];
      this.currentIndex = currentIndex;
      this.buf[0] = '*';         //mark the beginning of the buffer
      for (int i = 1; i <= currentIndex; ++i)
         this.buf[i] = '+';     // mark the full part of the buffer
      for (int i = currentIndex + 1; i <= SIZE; ++i)
         this.buf[i] = '0';      // mark the empty part of the buffer
      this.buf[SIZE + 1] = '*';  // mark the end of the buffer
   }


   public void put()
   {
      put('+');
   }


   public void put(char c)
   {
      ++currentIndex;
      Thread.yield(); // force race conditions to happen more often
      buf[currentIndex] = c;
   }


   public void get()
   {
      get('0');
   }


   public void get(char c)
   {
      buf[currentIndex] = c;
      Thread.yield(); // force race conditions to happen more often
      --currentIndex;
   }


   @Override
   public synchronized String toString()
   {
      String result = "";
      for (int i = 0; i < SIZE + 2; ++i)
      {
         result += buf[i];
      }
      result += "\n";
      for (int i = 0; i < this.currentIndex; ++i)
      {
         result += " ";
      }
      result += "^\n";
      return result;
   }
}
