/*

*/

/**
   This simulates a first-in-first-out (stack) buffer.
*/
public class Buffer_bounded extends Buffer
{
   private boolean empty;
   private boolean full;

   /**
      Set a default buffer size of 80.
   */
   Buffer_bounded(final int currentIndex)
   {
      this(currentIndex, 80);
   }

   /**
      Create a buffer that is bounded by using
      its internal monotor.
   */
   Buffer_bounded(final int currentIndex, final int SIZE)
   {
      super(currentIndex, SIZE); //Initialize a partially filled buffer.

      this.empty = (currentIndex == 0);
      this.full  = (currentIndex == SIZE);
   }

   @Override
   public synchronized void put(char c) // mutual exclusion
   {
      try
      {
         while (full)
         {
            wait();
         }
         ++currentIndex;
         Thread.yield(); // force race conditions to happen more often
         buf[currentIndex] = c;
         empty = (currentIndex == 0);
         full = (currentIndex == SIZE);
         notify(); // this MUST be notifyAll(), NOT notify()
         //System.out.println(buf);
      }
      catch (InterruptedException e)
      {
         // Without this line, the program can deadlock.
         Thread.currentThread().interrupt();
      }
   }


   @Override
   public synchronized void get(char c) // mutual exclusion
   {
      try
      {
         while (empty)
         {
            wait();
         }
         buf[currentIndex] = c;
         Thread.yield(); // force race conditions to happen more often
         --currentIndex;
         empty = (currentIndex == 0);
         full = (currentIndex == SIZE);
         notify(); // this MUST be notifyAll(), NOT notify()
         //System.out.println(buf);
      }
      catch (InterruptedException e)
      {
         // Without this line, the program can deadlock.
         Thread.currentThread().interrupt();
      }
   }
}
